/** Implémentation de la Calculette Capricieuse
 *
 * @see https://redmine.ryxeo.com/projects/leterrier-calc-cap/wiki
 * @author 2010 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
 * @author 2010 Francois Audirac <francois@webaf.net>
 * @see The GNU Public License (GPL)
 */

/**
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
* or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <iostream>
#include <QDebug>
#include <QObject>
#include <QtScript>
#include <QScriptEngine>
#include <QScrollBar>
#include "editeurtouche.h"
#include <QMessageBox>
#include "dlgsauve.h"
#include <QFileDialog>
#include <QFont>
#include <QLocale>
#include <QtGlobal>
#include <QKeyEvent>
#include "abuleduexercicev0.h"
#include "abuleduaproposv0.h"
#include <QLabel>

MainWindow::MainWindow(QWidget *parent) :
        AbulEduExerciceV0(parent),
        ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    ui->cbCompetences->hide();
    ui->cbNiveau->hide();
    ui->leSignature->hide();
    ui->lbTextSigne->hide();
    MainWindow::creeStatusBar(""); // Signature de la calculette
    nbReponses=0;
    modeEdition     = false; // La calculette démarre en mode "calculette"
    listeTouchesSaisies.append(ui->touche0); //La touche touche0 stocke 0 ou le résultat de l'opération précédente

    AbulEduAproposV0 *monAide=new AbulEduAproposV0(this);
    if(QFile::exists("./data/exercices/defaut.ini"))
    {
        //Il y a un fichier de paramètres par defaut : ./data/defaut.ini
        MainWindow::lisParametreDefaut();
    }
    else
    {
        /* Il n'y a pas de fichier ini par défaut donc on en crée un
           avec les valeurs données lors de la construction de l'interface
           Ceci ne devrait jamais se produire en utilisation normale!!! */
        MainWindow::ecrisParametreDefaut();

    }

    ui->touche0->hide();
    font24.setPointSize(24);
    font18.setPointSize(18);
    font12.setPointSize(12);
    font10.setPointSize(10);
    CSSDisabled="background-color: rgb(207, 207, 207);color: rgb(255, 255, 255);";
    locale= QLocale::QLocale();
    pointDecimal=locale.decimalPoint();
    MainWindow::installeFiltreEvenements();
    MainWindow::creeMenuCompetences();

    MainWindow::installeUtilisateur();
    // Création du menu exercices
    listeRepertoires[trUtf8("Mes exercices")]=m_chemin+"exercices"; //Dossier utilisateur
    listeRepertoires[trUtf8("Exercices communs")]="data/exercices"; //Dossier Application
    repertoireModeles[trUtf8("Modèles communs")]="data/modeles"; // Dossier des modèles communs
    repertoireModeles[trUtf8("Mes modèles")]=m_chemin+"modeles"; //Dossier des modèles perso


    MainWindow::creeArboMenuExo(listeRepertoires,ui->menuExercices);
    MainWindow::creeArboMenuExo(repertoireModeles,ui->menuCreerUneCalculetteAPartirDunModele);

}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}
// Les slots du clavier graphique
void MainWindow::on_touche00_clicked(){traiteTouche("touche00");}
void MainWindow::on_touche01_clicked(){traiteTouche("touche01");}
void MainWindow::on_touche02_clicked(){traiteTouche("touche02");}
void MainWindow::on_touche03_clicked(){traiteTouche("touche03");}
void MainWindow::on_touche04_clicked(){traiteTouche("touche04");}
void MainWindow::on_touche05_clicked(){traiteTouche("touche05");}
void MainWindow::on_touche06_clicked(){traiteTouche("touche06");}
void MainWindow::on_touche07_clicked(){traiteTouche("touche07");}
void MainWindow::on_touche08_clicked(){traiteTouche("touche08");}
void MainWindow::on_touche09_clicked(){traiteTouche("touche09");}
void MainWindow::on_touche10_clicked(){traiteTouche("touche10");}
void MainWindow::on_touche11_clicked(){traiteTouche("touche11");}
void MainWindow::on_touche12_clicked(){traiteTouche("touche12");}
void MainWindow::on_touche13_clicked(){traiteTouche("touche13");}
void MainWindow::on_touche14_clicked(){traiteTouche("touche14");}
void MainWindow::on_touche15_clicked(){traiteTouche("touche15");}
void MainWindow::on_touche16_clicked(){traiteTouche("touche16");}
void MainWindow::on_touche17_clicked(){traiteTouche("touche17");}
void MainWindow::on_touche18_clicked(){traiteTouche("touche18");}
void MainWindow::on_touche19_clicked(){traiteTouche("touche19");}
void MainWindow::on_touche20_clicked(){traiteTouche("touche20");}


//-------------------------------------------------------
//              Gestion des bouton et des actions
//-------------------------------------------------------

// Remet la ligne à zéro
void MainWindow::on_pushButtonEfface_clicked()
{
    MainWindow::creeNouvelleLigne("0");
}

// Réinitialise la calculette avec les paramètres chargés
void MainWindow::on_btnRAZ_clicked()
{
    MainWindow::sauveLog(); //On enregistre les log précédents
    //On cherche la liste des boutons contenant l'expression "touche"
    QRegExp nomBouton("touche");
    QList <QPushButton *> touches= ui->centralWidget->findChildren<QPushButton *>(nomBouton);
    //On remet le nombre d'utilisations possibles de la touche à sa valeur
    for (int i=0;i<touches.size();i++)
    {
        QVariant nb=touches[i]->property("nbFois");
        touches[i]->setProperty("nbFoisRestant",nb.toInt()); // Le Nombre de fois restant est égal
        touches[i]->setEnabled((nb.toInt()>0));              // au nombre de fois au démarrage
        if(touches[i]->isEnabled())
        {
            touches[i]->setStyleSheet("");
        }
        else
        {
            touches[i]->setStyleSheet(CSSDisabled);
        }
    }
    MainWindow::creeNouvelleLigne("0");

    MainWindow::effaceHistorique();
}

//-------------------------------------------------------
//              Gestion de la saisie
//-------------------------------------------------------
void MainWindow::traiteTouche(QString nomTouche)
{
    //Recherche du bouton ayant emis le signal avec findChild
    QPushButton *touche = ui->centralWidget->findChild<QPushButton *>(nomTouche);

    // Teste si on est en mode Edition
    if(modeEdition)
    {
        MainWindow::editeCalculette(touche);
    }
    else //Mode calculette
    {
        QString typeTouche = touche->property("typeTouche").toString();
        bool toucheModifiee=false;
        if (typeTouche=="memoire")
        {
            toucheModifiee=MainWindow::traiteToucheMemoire(touche);
        }
        else if(typeTouche=="chiffre")
        {
            //
            toucheModifiee=MainWindow::traiteToucheChiffre(touche);

        }
        else if(typeTouche=="operateur")
        {
            toucheModifiee=MainWindow::traiteToucheOperateur(touche);
        }
        else if(typeTouche=="(")
        {
            toucheModifiee=MainWindow::traiteToucheParentheseOuvrante(touche);
        }
        else if(typeTouche==")")
        {
            toucheModifiee=MainWindow::traiteToucheParentheseFermante(touche);
        }
        else if(typeTouche=="egale")
        {
            toucheModifiee=MainWindow::traiteToucheEgale();

        }
        if(toucheModifiee)
        {
            // Si le retour est true, on décrémente
            // et on met l'affichage à jour
            MainWindow::setAffichageLigneSaisie();
            MainWindow::decrementeNbFoisRestant(touche);
        }
    }
}

//-------------------------------------------------------
//        Utilitaires pour la gestion de la saisie
//-------------------------------------------------------

bool MainWindow::traiteToucheMemoire(QPushButton *touche)
{
    bool ok=false;
    if (!touche->text().startsWith("M")) // ALors la touche n'est pas affectée
    {
        if (listeTouchesSaisies.endsWith(ui->touche0))
            //On ne peut mettre une touche mémoire qu'en début de ligne
        {
            listeTouchesSaisies.removeLast();
            listeTouchesSaisies.append(touche);
            ok=true;
        }
        else if(listeTouchesSaisies.last()->property("typeTouche").toString()=="operateur")
            // Ou après un opérateur
        {
            listeTouchesSaisies.append(touche);
            ok=true;
        }
    }
    else
    {
        // Affecter la valeur à la touche uniquement si on est en début de ligne
        if(listeTouchesSaisies.size()==1
           && listeTouchesSaisies[0]->objectName()=="touche0")
        {
            touche->setText(ui->touche0->text());
            touche->setProperty("valeurEffective",ui->touche0->property("valeurEffective").toString());
        }
    }
    return ok;
}

bool MainWindow::traiteToucheChiffre(QPushButton *touche)
{
    bool ok=true;
    if(listeTouchesSaisies.last()->objectName()=="touche0")
    {
        listeTouchesSaisies.removeLast();
        listeTouchesSaisies.append(touche);
    }
    else if (listeTouchesSaisies.last()->property("typeTouche")!="memoire")
    {
        listeTouchesSaisies.append(touche);
    }
    return ok;
}

bool MainWindow::traiteToucheOperateur(QPushButton *touche)
{
    bool ok=false;
    if(listeTouchesSaisies.last()->property("typeTouche")=="operateur")
    {
        listeTouchesSaisies.removeLast();
        listeTouchesSaisies.append(touche);
        ok=true;
    }
    else
    {
        listeTouchesSaisies.append(touche);
        ok=true;
    }
    return ok;
}

bool MainWindow::traiteToucheParentheseOuvrante(QPushButton *touche)
{
    bool ok=false;
    if(listeTouchesSaisies.last()->objectName()=="touche0")
    {
        listeTouchesSaisies.prepend(touche);
        ok=true;
    }
    else if(listeTouchesSaisies.last()->property("typeTouche")=="chiffre"
            || listeTouchesSaisies.last()->property("typeTouche")=="memoire")
    {
        ok=false;
    }
    else
    {
        listeTouchesSaisies.append(touche);
        ok=true;
    }
    return ok;
}

bool MainWindow::traiteToucheParentheseFermante(QPushButton *touche)
{
    bool ok=false;
    if(listeTouchesSaisies.last()->property("typeTouche")=="chiffre"
       ||listeTouchesSaisies.last()->property("typeTouche")=="memoire" )
    {
        listeTouchesSaisies.append(touche);
        ok=true;
    }
    return ok;
}
bool MainWindow::traiteToucheEgale()
{
    // On respire un grand coup, va falloir analyser tout ça
    bool ok=false;
    QList<QPushButton *> ligneSaisies;
    ligneSaisies=listeTouchesSaisies; // On garde une copie de la saisie pour la restaurer au cas ou !
    QString expression="";
    QString membre="";

    while(!ligneSaisies.isEmpty())
    {
        if(ligneSaisies[0]->property("typeTouche")!="operateur")
        {
            membre=membre+ligneSaisies.takeFirst()->property("valeurEffective").toString();
        }
        else
        {
            membre=MainWindow::enleveZeros(membre);
            expression=expression+membre;
            expression=expression+ligneSaisies.takeFirst()->property("valeurEffective").toString();
            membre="";
        }
    }
    if(membre!="")
    {
        membre=MainWindow::enleveZeros(membre);
        expression=expression+membre;

        expression=MainWindow::calculeOperation(expression);
        if(expression=="nan") // Le calculateur retourne une erreur
        {
            expression=trUtf8("Erreur");
        }
        ui->touche0->setProperty("valeurEffective",expression);
        expression.replace(".",pointDecimal);
        MainWindow::afficheHistorique(expression);
        ui->touche0->setText(expression);
        listeTouchesSaisies.clear();
        listeTouchesSaisies.append(ui->touche0);
        nbReponses=nbReponses+1;
        setAbeLineLog("question", ui->afficheOperation->text()+"="+expression,-1,nbReponses);
        ok=true;
    }

    return ok;
}

void MainWindow::annuleToucheSaisie()
{
    //Consiste à effacer la dernière entrée dans listeTouchesSaisies
    //puis à rejouer la liste des touches
    if(!listeTouchesSaisies.isEmpty())      //On s'assure que la liste n'est pas vide
    {
        incrementeNbFoisRestant(listeTouchesSaisies.last());
        listeTouchesSaisies.removeLast();
    }


    if(listeTouchesSaisies.isEmpty())
    {
        listeTouchesSaisies.append(ui->touche0);
    }
    MainWindow::setAffichageLigneSaisie();
}

void MainWindow::decrementeNbFoisRestant(QPushButton* touche) // Diminue de 1 le nombre d'utilisations possibles de la touche
{

    QVariant nb=touche->property("nbFoisRestant");
    switch(nb.toInt())  //En fonction de nb : 0, 1, ou plus
    {
    case 0 : break;    //Il n'y a rien à faire
    case 1 :           //  On décrémente puis on désactive la touche
        nb=nb.toInt()-1;
        touche->setEnabled(false);
        touche->setStyleSheet(CSSDisabled);
        touche->setProperty("nbFoisRestant",nb);    //On enregistre la nouvelle valeur
        break;
    default :               // nbFoisRestant est supérieur à 1
            nb=nb.toInt()-1;    // On décrémente
}
    touche->setProperty("nbFoisRestant",nb);    //On enregistre la nouvelle valeur
    //listeTouchesSaisies.append(touche);//On ajoute la touche dans la liste des touches saisies

}

void MainWindow::incrementeNbFoisRestant(QPushButton *touche)
{
    QVariant nb=touche->property("nbFoisRestant");
    nb=nb.toInt()+1;    // On incrémente
    touche->setProperty("nbFoisRestant",nb);    //On enregistre la nouvelle valeur

}

void MainWindow::setAffichageLigneSaisie() // On met l'affichage à jour
{
    QString ligne = "";
    for(int i=0;i<listeTouchesSaisies.size();i++)
    {
        ligne=ligne+listeTouchesSaisies[i]->text();
    }
    ui->afficheOperation->setText(ligne);
}

QString MainWindow::calculeOperation(QString expression) // Évalue l'expression affichée
{

    // Evaluation du résultat

    QScriptEngine calculateur;                               //On crée un QScriptEngine
    QScriptValue resultat = calculateur.evaluate(expression);     //On évalue la ligne de calcul

    return QString::number(resultat.toNumber());             //On retourne la chaine correspondante
}

void MainWindow::afficheHistorique(QString resultat) //
{
    // Affichage du résultat dans l'historique
    resultat=resultat.replace(".",pointDecimal);
    QString ligne= ui->saisiesOperations->toHtml()
                   + ui->afficheOperation->text()+" = "
                   +"<font color=#FF0000 >"+ resultat+"</color>";      // On récupère le contenu de l'historique on ajoute le résultat en rouge
    ui->saisiesOperations->setHtml(ligne);                             // On affiche
    QScrollBar *sb = ui->saisiesOperations->verticalScrollBar();       //On scrolle vers le bas
    sb->setValue(sb->maximum());
}

void MainWindow::effaceHistorique()
{
    if(modeEdition)
    {
        ui->saisiesOperations->setHtml(trUtf8("Mode édition"));
    }
    else
    {
        ui->saisiesOperations->setHtml(trUtf8("Saisies des opérations"));
    }
}

void MainWindow::creeNouvelleLigne(QString resultat)        //On prépare la saisie d'une nouvelle ligne de calcul
{
    ui->touche0->setText(resultat);
    listeTouchesSaisies.clear();
    listeTouchesSaisies.append(ui->touche0);
}

QString MainWindow::enleveZeros(QString chaine) // Enlève les zéros en début de chaine
{
    QString parentheses="";
    while (chaine.startsWith("("))
    {
        parentheses=parentheses+"(";  //On enlève les éventuelles parenthèses
        chaine=chaine.remove(0,1);
    }

    while(chaine.startsWith("0") && chaine.size()>1)
    {
        chaine=chaine.remove(0,1);  // On enlève les zéros
    }
    chaine=parentheses+chaine; // On remet les parenthèses
    return chaine;
}

void MainWindow::activeTouches(bool etat)
{
    //On cherche la liste des boutons contenant l'expression "touche"
    QRegExp nomBouton("touche");
    QList <QPushButton *> touches= ui->centralWidget->findChildren<QPushButton *>(nomBouton);
    if(etat)
    {
        for(int i=0;i<touches.size();i++)
        {
            touches[i]->setEnabled(true);
            touches[i]->setStyleSheet("");
        }
    }
    else
    {
        //On remet le nombre d'utilisations possibles de la touche à sa valeur
        for (int i=0;i<touches.size();i++)
        {
            QVariant nb=touches[i]->property("nbFois");
            touches[i]->setProperty("nbFoisRestant",nb.toInt()); // Le Nombre de fois restant est égal
            touches[i]->setEnabled((nb.toInt()>0));              // au nombre de fois au démarrage
            if(touches[i]->isEnabled())
            {
                touches[i]->setStyleSheet("");
            }
            else
            {
                touches[i]->setStyleSheet(CSSDisabled);
            }
        }
    }
}

void MainWindow::changeTexte(QPushButton *touche, QString texte)
{
    switch (texte.size())
    {
    case 1:
        touche->setFont(font24);
        break;
    case 2:
        touche->setFont(font24);
        break;
    case 3:
        touche->setFont(font18);
        break;
    case 4:
        touche->setFont(font12);
        break;
    default:
        touche->setFont(font10);
        break;
    }
    touche->setText(texte);
}

void MainWindow::creeStatusBar(QString signature)
{
    lbNiveau=new QLabel(this);
    lbNiveau->setAlignment(Qt::AlignCenter);
    lbNiveau->setMinimumSize(lbNiveau->sizeHint());

    lbSignature=new QLabel(this);
    ui->statusBar->addWidget(lbNiveau,0);
    ui->statusBar->addWidget(lbSignature,1);
    lbSignature->setAlignment(Qt::AlignCenter);
    lbSignature->setMinimumSize(lbSignature->sizeHint());
    signature = trUtf8("Calculette créée par : ")+signature;
    lbSignature->setText(signature);
    lbNiveau->setText(niveauExo);
}

//-------------------------------------------------------
//              Editeur de touche
//-------------------------------------------------------

//Bascule mode édition/utilisation

void MainWindow::on_actionNouvelleCalculette_triggered()
{
    modeEdition=!modeEdition;
    if(modeEdition)
    {
        MainWindow::lisParametreDefaut();   //On charge la calculette par défaut
        setWindowTitle(trUtf8("Calculette sans nom")+"[*]");
        MainWindow::passeEnModeEdition();
    }
    else
    {
        MainWindow::revientEnModeCalcul();
        MainWindow::creeNouvelleLigne("0");
    }
}

void MainWindow::on_actionModifierCalculette_triggered()
{
    modeEdition=!modeEdition;
    MainWindow::passeEnModeEdition();
    if(modeEdition)
    {

        ui->actionModifierCalculette->setDisabled(true);
    }
}

void MainWindow::passeEnModeEdition()
{
    ui->actionNouvelleCalculette->setText(trUtf8("Utiliser la calculette")); //On modifie le menu
    ui->actionNouvelleCalculette->setProperty("shortcut","Ctrl+R");
    ui->actionNouvelleCalculette->setEnabled(true);
    ui->consignes->setReadOnly(false);       //On peut écrire dans le champs consignes
    ui->afficheOperation->setReadOnly(false);  //On peut écrire dans le champ saisieOperation
    ui->afficheOperation->setFont(font18);
    ui->afficheOperation->setAlignment(Qt::AlignCenter);
    ui->afficheOperation->setText(windowTitle().left(windowTitle().size()-3));  //on enlève "[*]"
    ui->centralWidget->setStyleSheet(CSSEditeur);
    MainWindow::activeTouches(true);        // Active les touches de façon a pouvoir les éditer
    ui->cbCompetences->show();
    ui->cbNiveau->show();
    ui->leSignature->show();
    ui->lbTextSigne->show();
    ui->pushButtonEfface->setDisabled(true);
    ui->btnRAZ->setDisabled(true);
}

void MainWindow::revientEnModeCalcul()
{
    if(MainWindow::isWindowModified())
    {
        ui->actionNouvelleCalculette->setDisabled(true);
        ui->actionModifierCalculette->setEnabled(true);
    }
    else
    {
        ui->actionNouvelleCalculette->setEnabled(true);
        ui->actionModifierCalculette->setEnabled(true);
    }
    ui->actionNouvelleCalculette->setText(trUtf8("Créer une nouvelle calculette..."));
    ui->actionNouvelleCalculette->setProperty("shortcut","Ctrl+N");
    ui->consignes->setReadOnly(true);      // On ne peut plus modifier les consignes
    ui->afficheOperation->setReadOnly(true);   //On ne peut plus écrire dans le champ saisieOperation
    ui->afficheOperation->setFont(font24);
    ui->afficheOperation->setAlignment(Qt::AlignRight);
    ui->centralWidget->setStyleSheet(CSSCalculette);
    MainWindow::activeTouches(false);  //Replace les touches dans leur état d'activation correct
    ui->saisiesOperations->setText(trUtf8("Saisie des opérations"));
    MainWindow::effaceHistorique();
    ui->cbCompetences->hide();
    ui->cbNiveau->hide();
    ui->leSignature->hide();
    ui->lbTextSigne->hide();
    ui->pushButtonEfface->setDisabled(false);
    ui->btnRAZ->setDisabled(false);

}
void MainWindow::editeCalculette(QPushButton *touche)
{
    QString type = touche->property("typeTouche").toString();
    QString valAff = touche->text();
    QString valEff = touche->property("valeurEffective").toString();
    int nb = touche->property("nbFois").toInt();
    EditeurTouche * fenetreEditeur = new EditeurTouche(type, valAff, valEff,nb,this);
    if(fenetreEditeur->exec())
    {
        QString texte=fenetreEditeur->valAffichee;
        MainWindow::changeTexte(touche,texte);
        touche->setProperty("valeurEffective", fenetreEditeur->valEffective);
        touche->setProperty("nbFois", fenetreEditeur->nb);
        touche->setProperty("nbFoisRestant", touche->property("nbFois"));
        touche->setProperty("typeTouche",fenetreEditeur->typeTouch);
        setWindowModified(true);
        ui->actionSauvegarderCalculette->setEnabled(true);
        ui->centralWidget->setStyleSheet(CSSEditeur);     //Applique les bonnes couleurs aux objets en fonction de leur style

    }
}

//-------------------------------------------------------
//              Sauvegarde/Restauration des paramètres de calculette
//-------------------------------------------------------
void MainWindow::ecrisParametreDefaut()     // Sauvegarde les paramètres par défaut
{
    QSettings parametres("./data/exercices/defaut.ini",QSettings::IniFormat);
    parametres.setValue("CSSCalculette",ui->centralWidget->styleSheet());
    parametres.setValue("CSSEditeur",ui->centralWidget->styleSheet()+"*{background-color: rgb(199, 24, 1);}");
    parametres.setValue("CSSDisabled","background-color: rgb(207, 207, 207);color: rgb(255, 255, 255);");
    // On commence la sauvegarde
    parametres.beginGroup("Touches");

    //On cherche la liste des boutons contenant l'expression "touche"
    QRegExp nomBouton("touche");
    QList <QPushButton *> touches= ui->centralWidget->findChildren<QPushButton *>(nomBouton);
    for (int i=0;i<touches.size();i++) //pour chaque touche
    {
        parametres.beginGroup(touches[i]->objectName());
        parametres.setValue("valeurAffichee", touches[i]->text());
        parametres.setValue("valeurEffective",touches[i]->property("valeurEffective"));
        parametres.setValue("typeTouche", touches[i]->property("typeTouche"));
        parametres.setValue("nbFois",touches[i]->property("nbFois"));
        parametres.endGroup();
    }
    parametres.endGroup();

    parametres.setValue("nom","Calculette Normale");

    parametres.setValue("consignes",ui->consignes->toHtml());  // On sauvegarde le champ de consigne au format Html

}

void MainWindow::lisParametreDefaut() // Restaure une calculette par défaut
{
    MainWindow::sauveLog();
    QSettings parametres("./data/exercices/defaut.ini",QSettings::IniFormat);
    CSSCalculette=parametres.value("CSSCalculette").toString();
    CSSEditeur=parametres.value("CSSEditeur").toString();
    CSSDisabled=parametres.value("CSSDisabled").toString();
    ui->centralWidget->setStyleSheet(CSSCalculette);
    ui->consignes->setHtml(parametres.value("consignes").toString());
    lbSignature->setText(trUtf8("Calculette créée par : ")+parametres.value("signature").toString());


    // On récupère les valeurs des paramètres des touches
    parametres.beginGroup("Defaut");
    QStringList listeTouches=parametres.childGroups(); // On récupère la liste des touches
    for(int i=0;i<listeTouches.size();i++)
    {
        parametres.beginGroup(listeTouches[i]);      // Pour chaque touche, on lui affecte ses paramètres
        QPushButton *touche=ui->centralWidget->findChild<QPushButton *>(listeTouches[i]);
        touche->setText(parametres.value("valeurAffichee").toString());
        touche->setProperty("valeurEffective",parametres.value("valeurEffective").toString());
        touche->setProperty("nbFois", parametres.value("nbFois").toInt());
        touche->setProperty("nbFoisRestant",parametres.value("nbFois").toInt()); // au début nbFoisRestant == nbFois
        touche->setEnabled((parametres.value("nbFois").toInt())>0);              // Active les touches en fonction du nombre de fois
        touche->setProperty("typeTouche",parametres.value("typeTouche").toString());
        parametres.endGroup();
    }
    parametres.endGroup();
    ui->centralWidget->setStyleSheet(CSSCalculette);
    MainWindow::activeTouches(false);
    setWindowTitle(parametres.value("nom").toString()+"[*]");
    ui->actionSauvegarderCalculette->setDisabled(true);
    lbSignature->setText(trUtf8("Calculette créée par : ")+parametres.value("signature").toString());
    niveauExo=parametres.value("niveau").toString();
    ui->cbNiveau->setCurrentIndex(ui->cbNiveau->findText(niveauExo));
    lbNiveau->setText(trUtf8("Niveau ")+niveauExo);
    competenceExo=parametres.value("competences").toString();
    ui->cbCompetences->setCurrentIndex(ui->cbCompetences->findText(competenceExo));
}

void MainWindow::on_actionCalculetteNormale_triggered()   // Restaure la calculette normale
{
    if(QFile::exists("./data/exercices/defaut.ini"))
    {
        MainWindow::lisParametreDefaut();

    }
    else
    {
        QMessageBox::information(this, trUtf8("Ouverture Calculette normale"),
                                 trUtf8("Le fichier de la calculette normale\n"
                                        "n'a pas été trouvé.\n"
                                        "L'opération est abandonnée."),
                                 QMessageBox::Yes | QMessageBox::Default);
    }
}

void MainWindow::on_actionSauvegarderCalculette_triggered()
{
    QString nomCalc=ui->afficheOperation->text(); // le nom "interne" de la calculette
    QString nomFichier = QFileDialog::getSaveFileName(this, trUtf8("Sauvegarde Calculette"),
                                                      m_chemin+"/"+nomCalc,
                                                      trUtf8("Calculettes (*.ini)"));

    MainWindow::sauveCalculette(nomFichier,nomCalc);
    setWindowModified(false);
    setWindowTitle(nomCalc+"[*]");
    // On actualise le menu des exercices
    MainWindow::creeArboMenuExo(listeRepertoires,ui->menuExercices);
}

void MainWindow::sauveCalculette(QString nomFichier, QString nomCalc)
{
    if(!nomFichier.contains(".ini")){nomFichier=nomFichier+".ini";}

    QSettings parametres(nomFichier,QSettings::IniFormat);

    // On commence la sauvegarde
    parametres.setValue("nom",nomCalc);
    parametres.beginGroup("Touches");

    //On cherche la liste des boutons contenant l'expression "touche"
    QRegExp nomBouton("touche");
    QList <QPushButton *> touches= ui->centralWidget->findChildren<QPushButton *>(nomBouton);
    for (int i=0;i<touches.size();i++) //pour chaque touche
    {
        parametres.beginGroup(touches[i]->objectName());
        parametres.setValue("valeurAffichee", touches[i]->text());
        parametres.setValue("valeurEffective",touches[i]->property("valeurEffective"));
        parametres.setValue("typeTouche", touches[i]->property("typeTouche"));
        parametres.setValue("nbFois",touches[i]->property("nbFois"));
        parametres.endGroup();
    }
    parametres.endGroup();
    parametres.setValue("consignes",ui->consignes->toHtml());  // On sauvegarde le champ de consigne au format Html
    parametres.setValue("signature", ui->leSignature->text());
    parametres.setValue("niveau", ui->cbNiveau->currentText());
    parametres.setValue("competences",ui->cbCompetences->currentIndex());

}

void MainWindow::chargeCalculette(QString nomFichier)
{

    MainWindow::sauveLog();
    MainWindow::effaceHistorique();
    QSettings parametres(nomFichier,QSettings::IniFormat);
    ui->consignes->setHtml(parametres.value("consignes").toString());
    lbSignature->setText(trUtf8("Calculette créée par : ")+parametres.value("signature").toString());
    niveauExo=parametres.value("niveau").toString();
    ui->cbNiveau->setCurrentIndex(ui->cbNiveau->findText(niveauExo));
    lbNiveau->setText(trUtf8("Niveau ")+niveauExo);
    competenceExo=parametres.value("competences").toString();
    ui->cbCompetences->setCurrentIndex(ui->cbCompetences->findText(competenceExo));
    // On récupère les valeurs des paramètres des touches
    parametres.beginGroup("Touches");
    QStringList listeTouches=parametres.childGroups(); // On récupère la liste des touches
    for(int i=0;i<listeTouches.size();i++)
    {
        parametres.beginGroup(listeTouches[i]);      // Pour chaque touche, on lui affecte ses paramètres
        QPushButton *touche=ui->centralWidget->findChild<QPushButton *>(listeTouches[i]);
        MainWindow::changeTexte(touche,parametres.value("valeurAffichee").toString());
        touche->setProperty("valeurEffective",parametres.value("valeurEffective").toString());
        touche->setProperty("nbFois", parametres.value("nbFois").toInt());
        touche->setProperty("nbFoisRestant",parametres.value("nbFois").toInt()); // au début nbFoisRestant == nbFois
        //touche->setEnabled((parametres.value("nbFois").toInt())>0);              // Active les touches en fonction du nombre de fois
        touche->setProperty("typeTouche",parametres.value("typeTouche").toString());
        parametres.endGroup();
    }
    parametres.endGroup();
    setWindowModified(false);
    setWindowTitle(parametres.value("nom").toString()+"[*]");
    ui->afficheOperation->setText(parametres.value("nom").toString());
    if(modeEdition)
    {
        ui->centralWidget->setStyleSheet(CSSEditeur);//Applique les bonnes couleurs aux objets en fonction de leur style
        MainWindow::activeTouches(true);
    }
    else
    {
        ui->centralWidget->setStyleSheet(CSSCalculette);// et du mode d'édition
        MainWindow::activeTouches(false);
        MainWindow::creeNouvelleLigne("0");
    }
    //
}

void MainWindow::on_actionChargerCalculette_triggered()
{
    QString nomFichier = QFileDialog::getOpenFileName(this, trUtf8("Sauvegarde Calculette"),
                                                      "data/exercices/",
                                                      trUtf8("Calculettes (*.ini)"));
    if(nomFichier!="")
    {
        MainWindow::chargeCalculette(nomFichier);
        ui->actionSauvegarderCalculette->setDisabled(true);
    }
}

//--------------------------------------------------------------------------------------
//
//------------------------------Souris sur les boutons--------------------------------
//
//--------------------------------------------------------------------------------------

bool MainWindow::eventFilter(QObject *obj, QEvent *event)
{
    if (event->type()==QEvent::KeyPress)
    {
        QKeyEvent *toucheClavier = static_cast<QKeyEvent *>(event);
        if(toucheClavier->key()==Qt::Key_Backspace && !modeEdition)
        {
            MainWindow::annuleToucheSaisie();
        }
    }
    if (obj->objectName().contains("touche") && modeEdition)  //ici on teste les boutons sur leur nom qui doit contenir touche et
    {
        if (event->type() == QEvent::Enter)  // La souris entre sur le bouton
        {
            int nb=obj->property("nbFois").toInt();
            QString nbfois="";
            if(nb>10)
            {
                nbfois=trUtf8("illimité");
            }
            else
            {
                nbfois=QString::number(nb);
            }
            QString type =obj->property("typeTouche").toString();
            if(type=="chiffre"){type=trUtf8("Chiffre");}
            else if(type=="memoire"){type=trUtf8("Mémoire");}
            else if(type=="operateur"){type=trUtf8("Opérateur");}
            else if(type=="egale"){type=trUtf8("Égale");}
            ui->saisiesOperations->setHtml(trUtf8("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
                                                  "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
                                                  "p, li { white-space: pre-wrap; }\n"
                                                  "</style></head><body style=\" font-family:'Sans'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
                                                  "<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Cliquez pour modifer les param\303\250tres de cette touche.</p>\n"
                                                  "<p align=\"center\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"></p>\n"
                                                  "<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Valeur actuelle des param\303\250tres</p>\n"
                                                  "<table border=\"0\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px;\" align=\"center\" cellspacing=\""
                                                  "2\" cellpadding=\"0\">\n"
                                                  "<tr>\n"
                                                  "<td>\n"
                                                  "<p align=\"right\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Valeur Affich\303\251e :</p></td>\n"
                                                  "<td>\n"
                                                  "<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" color:#ff0000;\">%1</span></p></td></tr>\n"
                                                  "<tr>\n"
                                                  "<td>\n"
                                                  "<p align=\"right\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Valeur Effective :</p></td>\n"
                                                  "<td>\n"
                                                  "<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" color:#ff0000;\">%2</span></p></td></tr>\n"
                                                  "<tr>\n"
                                                  "<td>\n"
                                                  "<p align=\"right\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Nombre de fois :</p></td>\n"
                                                  "<td>\n"
                                                  "<p style=\" margin-top:0px; margin-bottom:0p"
                                                  "x; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" color:#ff0000;\">%3</span></p></td></tr>\n"
                                                  "<tr>\n"
                                                  "<td>\n"
                                                  "<p align=\"right\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Type de touche :</p></td>\n"
                                                  "<td>\n"
                                                  "<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" color:#ff0000;\">%4</span></p></td></tr></table></body></html>")
                                           .arg(obj->property("text").toString())
                                           .arg(obj->property("valeurEffective").toString())
                                           .arg(nbfois)
                                           .arg(type));
            return true;
        }
        else if(event->type() == QEvent::Leave)  //La souris sort
        {
            ui->saisiesOperations->setText(trUtf8("Mode Édition"));
        }
        else
        {
            return false;
        }
    }
    else if(obj->objectName().contains("consignes")
        && modeEdition)
        {
        if (event->type() == QEvent::Enter)  // La souris entre sur le bouton
        {
            ui->saisiesOperations->setHtml(trUtf8("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
                                                  "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
                                                  "p, li { white-space: pre-wrap; }\n"
                                                  "</style></head><body style=\" font-family:'Sans'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
                                                  "<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Cliquez pour modifier les consignes </p>\n"
                                                  "<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">et modifiez le texte.</p>\n"
                                                  "<p align=\"justify\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"></p>\n"
                                                  "<p align=\"justify\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Il est aussi possible et plus ais\303\251 de r\303\251diger les "
                                                  "consignes dans un traitement de textes avec des effets de style (<span style=\" color:#ff0000;\">couleurs</span>, <span style=\" font-size:12pt;\">taille</span>, <span style=\" font-weight:600;\">gras</span>...) puis de les copier-coller dans ce cadre.</p></body></html>"));
        }
        else if(event->type() == QEvent::Leave)  //La souris sort
        {
            ui->saisiesOperations->setText(trUtf8("Mode Édition"));
        }
        else
        {
            return false;
        }

    }
    else if(obj->objectName().contains("afficheOperation")
        && modeEdition)
        {
        if (event->type() == QEvent::Enter)  // La souris entre sur le bouton

        {
            ui->saisiesOperations->setHtml(trUtf8("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
                                                  "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
                                                  "p, li { white-space: pre-wrap; }\n"
                                                  "</style></head><body style=\" font-family:'Sans'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
                                                  "<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Cliquez pour modifier le nom de la calculette </p>\n</body></html>"));
        }
        else if(event->type() == QEvent::Leave)  //La souris sort
        {
            ui->saisiesOperations->setText(trUtf8("Mode Édition"));
        }
        else
        {
            return false;
        }
    }
    else if(obj->objectName().contains("cbNiveau")
        && modeEdition)
        {
        if (event->type() == QEvent::Enter)  // La souris entre sur le bouton

        {
            ui->saisiesOperations->setHtml(trUtf8("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
                                                  "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
                                                  "p, li { white-space: pre-wrap; }\n"
                                                  "</style></head><body style=\" font-family:'Sans'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
                                                  "<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Sélectionnez le niveau de classe pour lequel cette calculette est créée. </p>\n</body></html>"));
        }
        else if(event->type() == QEvent::Leave)  //La souris sort
        {
            ui->saisiesOperations->setText(trUtf8("Mode Édition"));
        }
        else
        {
            return false;
        }
    }
    else if(obj->objectName().contains("cbCompetences")
        && modeEdition)
        {
        if (event->type() == QEvent::Enter)  // La souris entre sur le bouton

        {
            ui->saisiesOperations->setHtml(trUtf8("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
                                                  "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
                                                  "p, li { white-space: pre-wrap; }\n"
                                                  "</style></head><body style=\" font-family:'Sans'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
                                                  "<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Sélectionnez la compétence principale pour laquelle cette calculette est créée. </p>\n</body></html>"));
        }
        else if(event->type() == QEvent::Leave)  //La souris sort
        {
            ui->saisiesOperations->setText(trUtf8("Mode Édition"));
        }
        else
        {
            return false;
        }
    }
    else if(obj->objectName().contains("leSignature")
        && modeEdition)
        {
        if (event->type() == QEvent::Enter)  // La souris entre sur le bouton

        {
            ui->saisiesOperations->setHtml(trUtf8("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
                                                  "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
                                                  "p, li { white-space: pre-wrap; }\n"
                                                  "</style></head><body style=\" font-family:'Sans'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
                                                  "<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Indiquez ici le nom de l'auteur de cette calculette. </p>\n</body></html>"));
        }
        else if(event->type() == QEvent::Leave)  //La souris sort
        {
            ui->saisiesOperations->setText(trUtf8("Mode Édition"));
        }
        else
        {
            return false;
        }
    }
    {
        // On fait suivre l'event à la classe parent au cas ou il serait utilisé ailleurs
        return QMainWindow::eventFilter(obj, event);
    }
    {
        // On fait suivre l'event à la classe parent au cas ou il serait utilisé ailleurs
        return QMainWindow::eventFilter(obj, event);
    }
}


void MainWindow::installeFiltreEvenements()
{
    //On cherche la liste des boutons contenant l'expression "touche"
    QRegExp nomBouton("touche");
    QList <QPushButton *> touches= ui->centralWidget->findChildren<QPushButton *>(nomBouton);

    for (int i=0;i<touches.size();i++)
    {
        touches[i]->installEventFilter(this);
    }
    ui->consignes->installEventFilter(this);
    ui->centralWidget->installEventFilter(this);
    ui->afficheOperation->installEventFilter(this);
    ui->cbCompetences->installEventFilter(this);
    ui->cbNiveau->installEventFilter(this);
    ui->leSignature->installEventFilter(this);
}



void MainWindow::on_afficheOperation_editingFinished()
{
    ui->actionSauvegarderCalculette->setEnabled(true);
}

void MainWindow::on_consignes_textChanged()
{
    ui->actionSauvegarderCalculette->setEnabled(true);
}
// Teste arborescence utilisateur
void MainWindow::installeUtilisateur()
{
    m_home= QDir::home();
    m_chemin=m_home.absolutePath()+"/leterrier/calculette_capricieuse/data/exercices";


    if (!m_home.cd(m_chemin))

    {
        // On crée l'arborescence utilisateur de la calculette (le dossier exercices)
        m_dirTerrierData.mkpath(m_chemin);
        // On va créer le menu dynamique de l'utilisateur

    }
    m_home=QDir::home();
    m_chemin=m_home.absolutePath()+"/leterrier/calculette_capricieuse/data/modeles";
    if(!m_home.cd(m_chemin))
    {
        // On crée l'arborescence utilisateur de la calculette (le dossier des modèles)
        m_dirTerrierData.mkpath(m_chemin);
    }

    m_chemin=m_chemin.remove("modeles");

}

//Création de la combobox des compétences
void MainWindow::creeMenuCompetences()
{
    ui->cbCompetences->clear();

    QString locale = QLocale::system().name().section('_', 0, 0);
    QString fichier = "./data/competences/competences_"+locale+".txt";
    if(QFile::exists(fichier))
    {
        //Il y a un fichier de competences dans la locale
        QFile file(fichier);
             if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
                 return;
             while (!file.atEnd()) {
                 QString line = file.readLine();
                 ui->cbCompetences->addItem(line);
             }

    }
    else if(QFile::exists("./data/competences/competences_fr.txt"))
    {
        //Il y a un fichier de competences en français
        QFile file(fichier);
             if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
                 return;
             while (!file.atEnd()) {
                 QString line = file.readLine();
                 qDebug()<<line;
                 ui->cbCompetences->addItem(line);
             }

    }
    else
    {
        // Il n'y a pas de fichier de compétences : pas normal
        ui->cbCompetences->addItem("Pas de fichier de compétences");

    }
}

// Création dynamique du menu exercices

void MainWindow::creeArboMenuExo(QHash<QString,QString> listeRepertoires, QMenu * menuCourant)
{
    menuCourant->clear(); // On met le menu exercices à zéro
    QHashIterator<QString, QString> i(listeRepertoires); //itérateur pour parcourir la liste des répertoires
    while (i.hasNext())  //Tant qu'il y en a, on boucle
    {
        i.next();
        QMenu *  menucree=new QMenu(i.key(),this); //On crée un sous-menu avec le nom de la clé
        menu=menuCourant->addMenu(menucree);
        MainWindow::creeMenuExo(i.value(),menucree); // On crée l'arborescence du répertoire i.value()
    }
}

void MainWindow::creeMenuExo(QString repertoire, QMenu * menuCourant)
{
    //On crée un signalMapper qui établira le lien entre l'entrée de menu sélectionnée et le fichier à charger
    QSignalMapper *signalMapper = new QSignalMapper(this);
    connect(signalMapper, SIGNAL(mapped(QString)),this, SLOT(slotLanceExo(QString)));

    //On se positionne dans le répertoire des exercices
    QDir dir;
    dir.cd(repertoire);
    dir.setFilter(QDir::AllEntries | QDir::AllDirs | QDir::NoSymLinks | QDir::NoDotAndDotDot);
    dir.setSorting(QDir::DirsLast);
    //On crée un filtre pour ne conserver que les fichiers .ini
    QStringList filters;
    filters << "*.ini";
    dir.setNameFilters(filters);

    QFileInfoList list = dir.entryInfoList();
    for (int i = 0; i < list.size(); ++i) {
        QFileInfo fileInfo = list.at(i);

        if (fileInfo.isDir()) //Condition de sortie de la boucle de récursivité
        {
            //créer un sousmenu
            QMenu *  menucree=new QMenu(fileInfo.fileName(),this);
            menu=menuCourant->addMenu(menucree);
            MainWindow::creeMenuExo(fileInfo.filePath(),menucree);  //appel récursif
        }
        else
        {
            //créer un item de menu
            // Les lignes qui suivent sont adaptées à la façon de stocker les paramètres de calculette
            // On peut sinon utiliser :
            // QString nomItem = fileinfo.fileName();
            // qui affichera le nom du fichier débarassé de son path
            QSettings parametres(fileInfo.filePath(),QSettings::IniFormat); //On récupère le nom de la calculette
            QString nomItem = parametres.value("nom").toString();           //qui est caché dans le .ini
            QAction *act=menuCourant->addAction(nomItem);                   //et on l'affecte à l'item de menu
            // Reste à définir les actions elles-mêmes et à les connecter !!!
            signalMapper->setMapping(act,fileInfo.filePath());
            connect(act, SIGNAL(triggered()), signalMapper, SLOT(map()));

        }
    }
}

void MainWindow::closeEvent(QCloseEvent *event)
 {
     if (nbReponses>0)
    {
         MainWindow::sauveLog();
    }
     event->accept();
 }

void MainWindow::sauveLog()
{
    if(nbReponses>0)
    {
        setAbeExerciceName(windowTitle().remove("[*]"));
        setAbeLevel(niveauExo);
        setAbeSkill(competenceExo);
        pushAbulEduLogs();
        nbReponses=0;
    }
}

// Chargement de la calculette sélectionnée

void MainWindow::slotLanceExo(QString nomFichier)
{
    MainWindow::chargeCalculette(nomFichier);
}

void MainWindow::on_leSignature_textChanged(QString signature)
{
    lbSignature->setText(trUtf8("Calculette créée par : ")+signature);
}
