/** Implémentation de la Calculette Capricieuse
 *
 * @see https://redmine.ryxeo.com/projects/leterrier-calc-cap/wiki
 * @author 2010 Jean-Louis Frucot
 * @see The GNU Public License (GPL)
 */

/**
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
* or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QPushButton>
#include <QVector>
#include <QSettings>
#include <QStyle>
#include <QLocale>
#include <QDir>
#include "abuleduexercicev0.h"
#include "abuleduaproposv0.h"
#include <QLabel>

namespace Ui {
    class MainWindow;
}

class MainWindow : public AbulEduExerciceV0 {
    Q_OBJECT
public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();

protected:
    void changeEvent(QEvent *e);
    /** Filtre d'évènements utilisé pour le survol des boutons par la souris
      * @param obj un objet surveillé (de l'interface)
      * @param event le type d'évènement (ici Enter, Leave)
      */
    bool eventFilter(QObject *obj, QEvent *event);
private:
    Ui::MainWindow *ui;
    /**
      *C'est la fonction principale de gestion des touches du clavier graphique
      * @param nomTouche le nom de la touche cliquée
      *
      */
    //-------------------------------------------------------
    //        Variables
    //-------------------------------------------------------

    void traiteTouche(QString nomTouche);
    void traiteTouche2(QString nomTouche);

    /** Ouvre l'éditeur de touche pour la touche cliquée
      * @param touche la touche à éditer
      */
    void editeCalculette(QPushButton *touche);
    /**
      * \brief Décrémente le nombre de possibilités d'utilisation de la touche
      * @param touche pointeur vers un bouton
      *
      * Décrémente le nombre de possibilités d'utilisation de la touche.
      * Si le nombre de fois restant est 0 alors la touche est désactivée
      * Le nombre de fois restant est stocké dans une "propriété dynamique" du bouton :
      * "nbFoisRestant" qui est un QVariant<Int>
      */
    void decrementeNbFoisRestant(QPushButton *touche);


    /** Incrémente le nombre de possibilités d'utilisation de la touche.
      * Le nombre de fois restant est stocké dans une "propriété dynamique" du bouton :
      * "nbFoisRestant" qui est un QVariant<Int>
      */
    void incrementeNbFoisRestant(QPushButton *touche);

    /** Actualise l'affichage dans la ligne de saisie.
      */
    void setAffichageLigneSaisie();


    /** Gère l'appui sur une touche de type "memoire"
      * - si la touche affiche Mx alors on essaye de stocker le dernier résultat
      * - sinon on affiche la valeur de la touche à la place de l'opérande en cour
      * @param nomTouche le nom de la touche
      */
    QString getMemoire(QPushButton *touche);

    /** Installe les filtre d'évènement sur les boutons et champs concernés
      */
    void installeFiltreEvenements();


    QString CSSCalculette;
    QString CSSEditeur;
    QString CSSDisabled;
    QLocale locale;
    QString pointDecimal;
    QDir m_home;
    QDir m_dirTerrierData;
    QString m_chemin; //chemin vers dossier exo utilisateur
    QList<QPushButton *> listeTouchesSaisies;

    QLabel *lbSignature;
    QLabel *lbNiveau;
    int nbReponses;
    QString niveauExo;
    QString competenceExo;

    /** Détermine le mode de fonctionnement de la calculette
      * - true : mode utilisation
      * - false : mode édition
      */
    bool modeEdition;    // est à true quand on lance le programme


    /** Gère la saisie des opérateurs

      */
    bool traiteToucheOperateur(QPushButton *touche);

    bool traiteToucheMemoire(QPushButton *touche);

    bool traiteToucheChiffre(QPushButton *Touche);

    bool traiteToucheEgale();

    bool traiteToucheParentheseOuvrante(QPushButton *Touche);

    bool traiteToucheParentheseFermante(QPushButton *Touche);

    QFont font24;
    QFont font18;
    QFont font12;
    QFont font10;
    //-------------------------------------------------------
    //        Fonctions
    //-------------------------------------------------------

    /** Évalue l'expression saisie et visible dans l'afficheur
      * l'expression est évaluée de gauche à droite
      * @return la chaine représentant le résultat
      */
    QString calculeOperation(QString expression);

    /** Affiche la ligne de calcul dans l'historique
      * @param le resultat de l'opération
      */
    void afficheHistorique(QString resultat);

    /** Efface l'historique et affiche "Saisie des opérations" ou "Mode édition"
      */
    void effaceHistorique();

    /** Initialise une nouvelle ligne
      * @param resultat QString représentant le nombre à afficher au départ souvent le résultat ou "0"
      */
    void creeNouvelleLigne(QString resultat);

    /**Annule la dernière touche saisie (pas de limite en arrière)
      */
    void annuleToucheSaisie();

    /** Enregistre les paramètres par défaut de la calculette dans le fichier ./data/exercices/defaut.ini
      * Ces paramètres par défaut viennent de mainwindow.ui
      */
    void ecrisParametreDefaut();

    /** Restaure les paramètres par défaut de la calculette depuis le fichier ./data/exercices/defaut.ini
      *
      */
    void lisParametreDefaut();

    /** Enlève les zéros en début de chaine
      * @param chaine la chaine à traiter
      * @return chaine la chaine sans les zéros surnuméraires
      */
    QString enleveZeros(QString chaine);

    /** Active ou désactive les touches
      * @param etat vrai pour active tout, false active en fonction de nbFois
      */
    void activeTouches(bool etat);

    /**  Bascule en mode édition
     */
    void passeEnModeEdition();

    /** Revient en mode calcul
      */
    void revientEnModeCalcul();

    /** Sauvegarde les calculettes personnalisées
      * avec le nom "interne" de la calculette, les paramètres de touche
      * et le champs consignes
      * @param nomFichier le chemin du fichier
      * @param nomCalc le nom "interne" de la calculette il apparaitra dans les menus.
      */
    void sauveCalculette(QString nomFichier, QString nomCalc);

    /** Charge les calculettes personnalisées
      * @param nomFichier le chemin du fichier
      */
    void chargeCalculette(QString nomFichier);

    /** Modifie la taille du texte d'un bouton en tenant compte du nombre de caractères
      * @param texte le texte à afficher
      * @param touche le bouton à modifier
      */
    void changeTexte(QPushButton *touche, QString texte);

    /** Teste et crée si besoin l'arborescence utilisateur
      */
    void installeUtilisateur();
    /** Enregistre les logs sur le serveur
      */
    void sauveLog();

    /** Crée la liste des compétences dans le menu déroulant cbCompetences
      * Les competences sont disponibles dans un fichier texte localisé nommé competences_lang.txt
       */
    void creeMenuCompetences();

    /** Création dynamique du menu des exercices
      * @param listeRepertoire listes de chemins vers les répertoire des exercices
      * @param menu un pointeur vers un QMenu dans lequel sera installée l'arborescence des exercices
      */
    void creeArboMenuExo(QHash<QString,QString> listeRepertoires, QMenu * menuCourant);
    void creeMenuExo(QString repertoire,QMenu *menu);
    /** liste des répertoires de dossiers d'exercices et de leur intitulés */
    QHash<QString, QString> listeRepertoires;
    QHash<QString, QString> repertoireModeles;
    QAction * menu;
    QMenu * menuCourant;

    void creeStatusBar(QString signature);

signals:
    /** Demande le positionnement de l'onglet de l'aide
      * @param onglet : le numéro de l'onglet
      */
    void setindex(int onglet);


private slots:


    void on_leSignature_textChanged(QString );
    void on_consignes_textChanged();
    void on_afficheOperation_editingFinished();
    void on_actionModifierCalculette_triggered();
    void on_actionChargerCalculette_triggered();
    /** Slot de l'actionSauvegarderCalculette : ouvre un dialogue d'enregistrement de fichier
      */
    void on_actionSauvegarderCalculette_triggered();
    /** Slot de l'actionNouvelleCalculette : bascule entre mode Edition et mode Utilisation
      */
    void on_actionNouvelleCalculette_triggered();

    /** Demande la restauration des paramètres par défaut de la calculette
      */
    void on_actionCalculetteNormale_triggered();


    void on_touche20_clicked();
    void on_touche19_clicked();
    void on_touche18_clicked();
    void on_touche17_clicked();
    void on_touche16_clicked();
    void on_touche15_clicked();
    void on_touche14_clicked();
    void on_touche13_clicked();
    void on_touche12_clicked();
    void on_touche11_clicked();
    void on_touche10_clicked();
    void on_touche08_clicked();
    void on_touche07_clicked();
    void on_touche06_clicked();
    void on_touche05_clicked();
    void on_touche04_clicked();
    void on_touche03_clicked();
    void on_touche02_clicked();
    void on_touche01_clicked();
    void on_touche09_clicked();
    void on_touche00_clicked();

    void on_btnRAZ_clicked();

    /** Efface la ligne en cours de saisie et remets l'afficheur à zéro
      */
    void on_pushButtonEfface_clicked();


    /** Slot permettant de lancer un fichier d'exercice
      * @param nomFichier le chemein vers le fichier à charger
      */
    void slotLanceExo(QString nomFichier);

    void closeEvent(QCloseEvent *event);


};

#endif // MAINWINDOW_H
